#include	"header.h"

#include	<string.h>

/*
**	example relavant code stats here
**
*/

static void	read_property(Window, Atom, Atom, int);

static Atom	selection_property,
		multiple, *atoms, incr_property;


void init_eventhandler(void)
{
    char	*names[] = {
        "HOST_NAME", "U0",
        "FILE_NAME", "U1",
        "LINE_NUMBER", "U2",
        "NAME", "U3",
        "USER", "U4",
        "PROCEDURE", "U5",
        "CLASS", "U6",
        "POSTSCRIPT", "U7",
        "TARGETS", "U8",
        "STRING", "U9"
    };
    
    selection_property = XInternAtom (dpy, "TEST_PROP", False);
    multiple = XInternAtom(dpy, "MULTIPLE", False);
    incr_property = XInternAtom(dpy, "INCR", False);

    atoms = (Atom*)malloc(20 * sizeof(Atom));

    if (!XInternAtoms(dpy, names, 20, False, atoms))
    {
        fprintf(stderr, "Couldn't get the Atoms.\n");
        exit(1);
    }
   
}

int selection_notify(XSelectionEvent ev)
/* This function is passed the event that was
** triggered if it is a SelectionNotify type. */
{
	char	*buffer, *buff_p;
	int	format, i;
	Atom	type_return, *atoms;

	unsigned char	*data;
	unsigned long	nitems, bytes_left, total;


	if (ev.property == None)
		return 1; /* The selection failed. */

	printf("Property: %s\n", XGetAtomName(dpy, ev.property));

        XGetWindowProperty(dpy, ev.requestor, ev.property,
                           0L, 4096L, False, AnyPropertyType,
                           &type_return, &format, &nitems,
                           &bytes_left, &data);

        printf("Type: %s\n", XGetAtomName(dpy, type_return));
        printf("Format: %d\n", format);
        printf("Item Count: %d\n", nitems);

        atoms = (Atom*)data;
        printf("-----Property Contents----\n");
        for (i = 0; i < 20; i += 2)
            if (atoms[i] != None)
                read_property(ev.requestor, atoms[i], atoms[i+1], i);

        XFree(data);
        XDeleteProperty(dpy, ev.requestor, ev.property);
        XFlush(dpy);
        
        return 1;
}

static void read_property(Window	win,
                          Atom 		target,
                          Atom 		property,
                          int 		locator)
{
    char	*buffer, *buff_p;
    int		format, i;
    Atom	type_return, *atoms;

    unsigned char	*data;
    unsigned long	nitems, bytes_left, total;

    if (property == None)
    {
        /* TK applications seem to be broken in this way
        ** so, if you don't check, you liable to explode. */
        fprintf(stderr, "Broken owner!\nTarget: %s\n",
                XGetAtomName(dpy, target));
        return;
    }

    XGetWindowProperty(dpy, win, property, 0L, 4096L,
                       False, AnyPropertyType, &type_return,
                       &format, &nitems, &bytes_left, &data);

    buff_p = buffer = (char*)calloc(nitems);
    memcpy(buffer, data, nitems);
    buff_p += nitems;
    XFree(data);

    for (total = nitems;bytes_left;)
    {
        XGetWindowProperty(dpy, win, property, (total/4), 4096,
                           False, AnyPropertyType, &type_return,
                           &format, &nitems, &bytes_left, &data);


        total += nitems;
        realloc(buffer, total);
        memset(buff_p, '\0', nitems);

        memcpy(buff_p, data, nitems);
        XFree(data);
        buff_p += nitems;
    }

    printf("Item Count: %d\n", total);

    switch (locator)
    {
        case 0:
        case 2:
        case 6:
        case 8:
        case 10:
        case 14:
            if (format != 8)
                break;
            printf("%s: %s\n", XGetAtomName(dpy, target), buffer);
            break;
        case 12:
            
            printf("CLASS: %s, %s\n", buffer,
                   (buff_p = (buffer + strlen(buffer) + 1)));
            break;
        case 4:
            printf("LINE_NUMBER\n");
            printf("Format: %d\n", format);
            break;
        case 18:
            printf("STRING:\n%s\n\n\n", buffer);
            break;
        case 16:
            printf("Targets:\n");
            for (i = 0, atoms = (Atom*)buffer; i < total; i++, atoms++)
                if (*atoms == None) printf("None\n");
                else
                    fprintf(stderr, "\t%s\n", XGetAtomName(dpy, *atoms));
            break;

        default:
            printf("You miscounted!\n");
            printf("locator: %d\n", locator);
    }

    if (type_return == incr_property)
        printf("INCR selection\n");
    XDeleteProperty(dpy, win, property);
    XFlush(dpy);
    free(buffer);
}

int buttonPress(XButtonEvent ev)
{
    if (ev.button == 2)
    {
        XChangeProperty(dpy, top_window, selection_property, XA_ATOM,
                        32, PropModeReplace, (unsigned char*)atoms,
                        20);

        XConvertSelection(dpy, XA_PRIMARY, multiple, selection_property,
                          ev.window, ev.time);
    }
    return 1;
}


/*
**	Example relavant code ends here.
**
*/





int clientMessage(XClientMessageEvent ev)
{
    if (ev.format == 32 && ev.data.l[0] == delete_window)
    {
        printf("Delete window request\n");
        return 0;
    }

    printf("Unknown client message event\n");
    return 1;
}



int selectionRequest(XSelectionRequestEvent ev)
{
    return 1;
}

int propertyNotify(XPropertyEvent ev)
{
    return 1;
}


int buttonRelease(XButtonEvent ev)
{
    return 1;
}

int expose(XExposeEvent ev)
{
    return 1;
}

int configureNotify(XConfigureEvent ev)
{
    return 1;
}

int motion(XMotionEvent ev)
{
    return 1;
}
